@@ -1,41 +1,44 @@
-exports.Form = Form;
+var stream = require('stream');
+var util = require('util');
+var fs = require('fs');
+var crypto = require('crypto');
+var path = require('path');
+var os = require('os');
+var StringDecoder = require('string_decoder').StringDecoder;
+var FdSlicer = require('fd-slicer');
 
-var stream = require('readable-stream')
-  , util = require('util')
-  , fs = require('fs')
-  , crypto = require('crypto')
-  , path = require('path')
-  , os = require('os')
-  , StringDecoder = require('string_decoder').StringDecoder
-  , StreamCounter = require('stream-counter')
+var START = 0;
+var START_BOUNDARY = 1;
+var HEADER_FIELD_START = 2;
+var HEADER_FIELD = 3;
+var HEADER_VALUE_START = 4;
+var HEADER_VALUE = 5;
+var HEADER_VALUE_ALMOST_DONE = 6;
+var HEADERS_ALMOST_DONE = 7;
+var PART_DATA_START = 8;
+var PART_DATA = 9;
+var PART_END = 10;
+var CLOSE_BOUNDARY = 11;
+var END = 12;
 
-var START = 0
-  , START_BOUNDARY = 1
-  , HEADER_FIELD_START = 2
-  , HEADER_FIELD = 3
-  , HEADER_VALUE_START = 4
-  , HEADER_VALUE = 5
-  , HEADER_VALUE_ALMOST_DONE = 6
-  , HEADERS_ALMOST_DONE = 7
-  , PART_DATA_START = 8
-  , PART_DATA = 9
-  , PART_END = 10
-  , CLOSE_BOUNDARY = 11
-  , END = 12
-
-  , LF = 10
-  , CR = 13
-  , SPACE = 32
-  , HYPHEN = 45
-  , COLON = 58
-  , A = 97
-  , Z = 122
+var LF = 10;
+var CR = 13;
+var SPACE = 32;
+var HYPHEN = 45;
+var COLON = 58;
+var A = 97;
+var Z = 122;
 
 var CONTENT_TYPE_RE = /^multipart\/(?:form-data|related)(?:;|$)/i;
 var CONTENT_TYPE_PARAM_RE = /;\s*([^=]+)=(?:"([^"]+)"|([^;]+))/gi;
 var FILE_EXT_RE = /(\.[_\-a-zA-Z0-9]{0,16}).*/;
 var LAST_BOUNDARY_SUFFIX_LEN = 4; // --\r\n
 
+// replace base64 characters with safe-for-filename characters
+var b64Safe = {'/': '_', '+': '-'};
+
+exports.Form = Form;
+
 util.inherits(Form, stream.Writable);
 function Form(options) {
   var self = this;
@@ -44,7 +47,6 @@ function Form(options) {
   options = options || {};
 
   self.error = null;
-  self.finished = false;
 
   self.autoFields = !!options.autoFields;
   self.autoFiles = !!options.autoFiles;
@@ -54,7 +56,6 @@ function Form(options) {
   self.maxFilesSize = options.maxFilesSize || Infinity;
   self.uploadDir = options.uploadDir || os.tmpDir();
   self.encoding = options.encoding || 'utf8';
-  self.hash = options.hash || false;
 
   self.bytesReceived = 0;
   self.bytesExpected = null;
@@ -68,7 +69,7 @@ function Form(options) {
   self.backpressure = false;
   self.writeCbs = [];
 
-  if (options.boundary) setUpParser(self, options.boundary);
+  self.emitQueue = [];
 
   self.on('newListener', function(eventName) {
     if (eventName === 'file') {
@@ -89,9 +90,8 @@ Form.prototype.parse = function(req, cb) {
     self.autoFields = true;
     self.autoFiles = true;
 
-    var fields = {};
-    var files = {};
-    self.on('error', function(err) {
+    // wait for request to end before calling cb
+    var end = function (done) {
       if (called) return;
 
       called = true;
@@ -101,12 +101,18 @@ Form.prototype.parse = function(req, cb) {
         if (waitend && req.readable) {
           // dump rest of request
           req.resume();
-          req.once('end', function() {
-            cb(err);
-          });
+          req.once('end', done);
           return;
         }
 
+        done();
+      });
+    };
+
+    var fields = {};
+    var files = {};
+    self.on('error', function(err) {
+      end(function() {
         cb(err);
       });
     });
@@ -119,7 +125,9 @@ Form.prototype.parse = function(req, cb) {
       filesArray.push(file);
     });
     self.on('close', function() {
-      cb(null, fields, files);
+      end(function() {
+        cb(null, fields, files);
+      });
     });
   }
 
@@ -185,12 +193,12 @@ Form.prototype.parse = function(req, cb) {
       self.error = err;
       req.removeListener('aborted', onReqAborted);
       req.removeListener('end', onReqEnd);
+      if (self.destStream) {
+        self.destStream.emit('error', err);
+      }
     }
 
-    self.openedFiles.forEach(function(file) {
-      destroyFile(self, file);
-    });
-    self.openedFiles = [];
+    cleanupOpenFiles(self);
 
     if (first) {
       self.emit('error', err);
@@ -206,20 +214,20 @@ Form.prototype.parse = function(req, cb) {
 Form.prototype._write = function(buffer, encoding, cb) {
   if (this.error) return;
 
-  var self = this
-    , i = 0
-    , len = buffer.length
-    , prevIndex = self.index
-    , index = self.index
-    , state = self.state
-    , lookbehind = self.lookbehind
-    , boundary = self.boundary
-    , boundaryChars = self.boundaryChars
-    , boundaryLength = self.boundary.length
-    , boundaryEnd = boundaryLength - 1
-    , bufferLength = buffer.length
-    , c
-    , cl
+  var self = this;
+  var i = 0;
+  var len = buffer.length;
+  var prevIndex = self.index;
+  var index = self.index;
+  var state = self.state;
+  var lookbehind = self.lookbehind;
+  var boundary = self.boundary;
+  var boundaryChars = self.boundaryChars;
+  var boundaryLength = self.boundary.length;
+  var boundaryEnd = boundaryLength - 1;
+  var bufferLength = buffer.length;
+  var c;
+  var cl;
 
   for (i = 0; i < len; i++) {
     c = buffer[i];
@@ -504,16 +512,12 @@ Form.prototype.onParseHeadersEnd = function(offset) {
       self.boundary.length - LAST_BOUNDARY_SUFFIX_LEN) :
     undefined;
 
-  self.emit('part', self.destStream);
   if (self.destStream.filename == null && self.autoFields) {
     handleField(self, self.destStream);
   } else if (self.destStream.filename != null && self.autoFiles) {
     handleFile(self, self.destStream);
   } else {
-    beginFlush(self);
-    self.destStream.on('end', function(){
-      endFlush(self);
-    });
+    handlePart(self, self.destStream);
   }
 }
 
@@ -542,76 +546,122 @@ function beginFlush(self) {
 
 function endFlush(self) {
   self.flushing -= 1;
+
+  if (self.flushing < 0) {
+    // if this happens this is a critical bug in multiparty and this stack trace
+    // will help us figure it out.
+    self.handleError(new Error("unexpected endFlush"));
+    return;
+  }
+
   maybeClose(self);
 }
 
 function maybeClose(self) {
-  if (!self.flushing && self.finished && !self.error) {
-    self.emit('close');
+  if (self.flushing > 0 || self.error) return;
+
+  // go through the emit queue in case any field, file, or part events are
+  // waiting to be emitted
+  holdEmitQueue(self)(function() {
+    // nextTick because the user is listening to part 'end' events and we are
+    // using part 'end' events to decide when to emit 'close'. we add our 'end'
+    // handler before the user gets a chance to add theirs. So we make sure
+    // their 'end' event fires before we emit the 'close' event.
+    // this is covered by test/standalone/test-issue-36
+    process.nextTick(function() {
+      self.emit('close');
+    });
+  });
+}
+
+function cleanupOpenFiles(self) {
+  self.openedFiles.forEach(function(internalFile) {
+    // since fd slicer autoClose is true, destroying the only write stream
+    // is guaranteed by the API to close the fd
+    internalFile.ws.destroy();
+
+    fs.unlink(internalFile.publicFile.path, function(err) {
+      if (err) self.handleError(err);
+    });
+  });
+  self.openedFiles = [];
+}
+
+function holdEmitQueue(self) {
+  var o = {cb: null};
+  self.emitQueue.push(o);
+  return function(cb) {
+    o.cb = cb;
+    flushEmitQueue(self);
+  };
+}
+
+function flushEmitQueue(self) {
+  while (self.emitQueue.length > 0 && self.emitQueue[0].cb) {
+    self.emitQueue.shift().cb();
   }
 }
 
-function destroyFile(self, file) {
-  if (!file.ws) return;
-  file.ws.removeAllListeners('close');
-  file.ws.on('close', function() {
-    fs.unlink(file.path, function(err) {
-      if (err && !self.error) self.handleError(err);
-    });
+function handlePart(self, partStream) {
+  beginFlush(self);
+  var emitAndReleaseHold = holdEmitQueue(self);
+  partStream.on('end', function() {
+    endFlush(self);
+  });
+  emitAndReleaseHold(function() {
+    self.emit('part', partStream);
   });
-  file.ws.destroy();
 }
 
 function handleFile(self, fileStream) {
   if (self.error) return;
-  var file = {
+  var publicFile = {
     fieldName: fileStream.name,
     originalFilename: fileStream.filename,
     path: uploadPath(self.uploadDir, fileStream.filename),
     headers: fileStream.headers,
+    size: 0,
+  };
+  var internalFile = {
+    publicFile: publicFile,
+    ws: null,
   };
   beginFlush(self); // flush to write stream
-  file.ws = fs.createWriteStream(file.path);
-  self.openedFiles.push(file);
-  fileStream.pipe(file.ws);
-  var counter = new StreamCounter();
-  var seenBytes = 0;
-  fileStream.pipe(counter);
-  var hashWorkaroundStream
-    , hash = null;
-  if (self.hash) {
-    // workaround stream because https://github.com/joyent/node/issues/5216
-    hashWorkaroundStream = stream.Writable();
-    hash = crypto.createHash(self.hash);
-    hashWorkaroundStream._write = function(buffer, encoding, callback) {
-      hash.update(buffer);
-      callback();
-    };
-    fileStream.pipe(hashWorkaroundStream);
-  }
-  counter.on('progress', function() {
-    var deltaBytes = counter.bytes - seenBytes;
-    seenBytes += deltaBytes;
-    self.totalFileSize += deltaBytes;
-    if (self.totalFileSize > self.maxFilesSize) {
-      if (hashWorkaroundStream) fileStream.unpipe(hashWorkaroundStream);
-      fileStream.unpipe(counter);
-      fileStream.unpipe(file.ws);
-      self.handleError(new Error("maxFilesSize " + self.maxFilesSize + " exceeded"));
-    }
+  var emitAndReleaseHold = holdEmitQueue(self);
+  fileStream.on('error', function(err) {
+    self.handleError(err);
   });
-  file.ws.on('error', function(err) {
-    if (!self.error) self.handleError(err);
-  });
-  file.ws.on('close', function() {
-    if (hash) file.hash = hash.digest('hex');
-    file.size = counter.bytes;
-    self.emit('file', fileStream.name, file);
-    endFlush(self);
-  });
-  beginFlush(self); // flush from file stream
-  fileStream.on('end', function(){
-    endFlush(self);
+  fs.open(publicFile.path, 'w', function(err, fd) {
+    if (err) return self.handleError(err);
+    var fdSlicer = new FdSlicer(fd, {autoClose: true});
+
+    // end option here guarantees that no more than that amount will be written
+    // or else an error will be emitted
+    internalFile.ws = fdSlicer.createWriteStream({end: self.maxFilesSize - self.totalFileSize});
+
+    // if an error ocurred while we were waiting for fs.open we handle that
+    // cleanup now
+    self.openedFiles.push(internalFile);
+    if (self.error) return cleanupOpenFiles(self);
+
+    var prevByteCount = 0;
+    internalFile.ws.on('error', function(err) {
+      self.handleError(err);
+    });
+    internalFile.ws.on('progress', function() {
+      publicFile.size = internalFile.ws.bytesWritten;
+      var delta = publicFile.size - prevByteCount;
+      self.totalFileSize += delta;
+      prevByteCount = publicFile.size;
+    });
+    fdSlicer.on('close', function() {
+      if (self.error) return;
+      emitAndReleaseHold(function() {
+        self.emit('file', fileStream.name, publicFile);
+      });
+      endFlush(self);
+    });
+    fileStream.pipe(internalFile.ws);
   });
 }
 
@@ -620,6 +670,10 @@ function handleField(self, fieldStream) {
   var decoder = new StringDecoder(self.encoding);
 
   beginFlush(self);
+  var emitAndReleaseHold = holdEmitQueue(self);
+  fieldStream.on('error', function(err) {
+    self.handleError(err);
+  });
   fieldStream.on('readable', function() {
     var buffer = fieldStream.read();
     if (!buffer) return;
@@ -633,7 +687,9 @@ function handleField(self, fieldStream) {
   });
 
   fieldStream.on('end', function() {
-    self.emit('field', fieldStream.name, value);
+    emitAndReleaseHold(function() {
+      self.emit('field', fieldStream.name, value);
+    });
     endFlush(self);
   });
 }
@@ -665,26 +721,35 @@ function setUpParser(self, boundary) {
   self.index = null;
   self.partBoundaryFlag = false;
 
+  beginFlush(self);
   self.on('finish', function() {
-    if ((self.state === HEADER_FIELD_START && self.index === 0) ||
-        (self.state === PART_DATA && self.index === self.boundary.length))
-    {
-      self.onParsePartEnd();
-    } else if (self.state !== END) {
+    if (self.state !== END) {
       self.handleError(new Error('stream ended unexpectedly'));
     }
-    self.finished = true;
-    maybeClose(self);
+    endFlush(self);
   });
 }
 
 function uploadPath(baseDir, filename) {
   var ext = path.extname(filename).replace(FILE_EXT_RE, '$1');
-  var name = process.pid + '-' +
-    (Math.random() * 0x100000000 + 1).toString(36) + ext;
+  var name = randoString(18) + ext;
   return path.join(baseDir, name);
 }
 
+function randoString(size) {
+  return rando(size).toString('base64').replace(/[\/\+]/g, function(x) {
+    return b64Safe[x];
+  });
+}
+
+function rando(size) {
+  try {
+    return crypto.randomBytes(size);
+  } catch (err) {
+    return crypto.pseudoRandomBytes(size);
+  }
+}
+
 function parseFilename(headerValue) {
   var m = headerValue.match(/\bfilename="(.*?)"($|; )/i);
   if (!m) {
