@@ -1,10 +1,13 @@
-var multiparty = require('../../')
-  , assert = require('assert')
-  , http = require('http')
-  , net = require('net')
-  , stream = require('readable-stream');
+var multiparty = require('../../');
+var assert = require('assert');
+var http = require('http');
+var net = require('net');
+var Pend = require('pend');
+var stream = require('stream');
 
+var err = null;
 var form = new multiparty.Form();
+var pend = new Pend();
 var req = new stream.Readable();
 var unpiped = false;
 
@@ -15,17 +18,25 @@ req._read = function(){
   this.push(new Buffer('--foo!'));
 };
 
-form.on('error', function(err){
+pend.go(function(){
+  form.on('error', function(e){
+    err = e;
+  });
+});
+
+pend.go(function(){
+  form.on('unpipe', function(){
+    unpiped = true;
+  });
+});
+
+pend.wait(function(){
   // verification that error event implies unpipe call
   assert.ok(err);
   assert.ok(unpiped, 'req was unpiped');
   assert.equal(req._readableState.flowing, false, 'req not flowing');
   assert.equal(req._readableState.pipesCount, 0, 'req has 0 pipes');
-});
-
-form.on('unpipe', function(){
-  unpiped = true;
-});
+})
 
 form.parse(req)
 assert.equal(req._readableState.flowing, true, 'req flowing');
